
local paths = require('saotrace.suplib.paths')

describe( "split", function ()

  local tests = { ["a/b"] = { dirname = 'a', basename = 'b' },
		  ["b"]   = { dirname = nil, basename = 'b' },
		  ["/"]   = { dirname = "", basename = "" },
		  ["a/"]  = { dirname = "a", basename = "" },
		  ["/b"]  = { dirname = "", basename = "b" },
		  ["a/b/c"]  = { dirname = "a/b", basename = "c" },
		  ["a/b/"]  = { dirname = "a/b", basename = "" },
	       }

  for path, exp in pairs(tests) do

    it( path, function ()
      dirname, basename = paths.split( path )
      assert.are.same( exp, { dirname = dirname, basename = basename } )
    end)

  end

end)

describe( "is_modname", function ()

  describe( 'pass', function ()
    local tests = { 'a.b.c', 'a' }

    for _,v in pairs( tests ) do

       it( v, function ()
         assert.is_true( paths.is_modname( v ) )
       end)

    end
  end)

  describe( 'fail', function ()
    local tests = { '.c', 'a.', 'a-b.c', 'a.b-c' }

    for _,v in pairs( tests ) do

       it( v, function ()
         assert.is_not_true( paths.is_modname( v ) )
       end)

    end
  end)


end)


