
local tables = require('saotrace.suplib.tables' )

describe( "copy", function ()

  describe( "shallow copy", function ()

    b = { 1, 2 }
    a = { x = 1, y = 2, z = b }

    copy = tables.copy( a )

    it( "contents", function ()
      assert.are.same( a, copy )
    end)

    it( "copy references b", function ()
      b[1] = 3
      assert.is.equal( 3, copy.z[1] )
    end)

  end)


  describe( "deep copy", function ()

    b = { 1, 2 }
    a = { x = 1, y = 2, z = b }

    copy = tables.copy( a, 1 )

    it( "contents", function ()
      assert.are.same( a, copy )
    end)

    it( "copy doesn't reference b", function ()
      b[1] = 3
      assert.is.equal( 1, copy.z[1] )
    end)

  end)

end)

describe( "merge", function ()

  local merge = require( 'saotrace.suplib.tables' ).merge

  it( "merge 3 tables", function ()

     local exp = { a = 1, b = 2, c = 3 }

     local tbl = merge( nil, { a = 1 }, { b = 2 }, { c = 3 } )

     assert.are.same( exp, tbl )

  end)

  it( "deep compare", function ()

    local exp = { a = 1, b = 2, c = 3, d = { e = 4, f = 5 } }

    local tbl = merge( nil,
		       { a = 1 },
		       { b = 2 },
		       { c = 3 },
		       { d = { e = 4 } },
		       { d = { f = 5 } }
		    )
    assert.are.same( exp, tbl )

  end)

end)

describe( "extract", function ()

  local extract = require( 'saotrace.suplib.tables' ).extract

  it( "none", function ()

    local exp = { a = 1, b = 2, c = 3 }

    local tbl = extract( exp )

    assert.are.same( exp, tbl )

  end)

  it( "keys", function ()

    local exp = { b = 2, c = 3 }

    local tbl = extract( { a = 1, b = 2, c = 3 }, { 'b', 'c' } )

    assert.are.same( exp, tbl )
  end)

  it( "map", function ()

    local exp = { b = 2, c = 3 }

    local tbl = extract( { d = 2, e = 3 }, nil, { b = 'd', c = 'e' } )

    assert.are.same( exp, tbl )

  end)

  it( "keys_map", function ()

    local exp = { d = 2 }

    local tbl = extract( { b = 2, c = 3 }, { 'd' } , { ['d'] = 'b', ['e'] = 'c' } )

    assert.are.same( exp, tbl )

  end)

end)







